<?php

namespace Drupal\ekan_visuals\Plugin\EkanVisualisation;

use Drupal\Core\Form\FormStateInterface;
use Drupal\Component\Utility\Html;
use Drupal\ekan_core\Entity\EkanResourceEntity;
use Drupal\file\FileInterface;
use Drupal\leaflet\LeafletSettingsElementsTrait;

/**
 * A Geo map visualisation based on uploaded file.
 *
 * @EkanVisualisation(
 *   id = "geojson_map",
 *   label = @Translation("Geojson Map"),
 *   formats = {
 *   "geojson"
 *   }
 * )
 */
class GeojsonMap extends EkanVisualisationBase {

  use LeafletSettingsElementsTrait;

  /**
   * {@inheritdoc}
   */
  public static function defaultSettings() {
    return [];
  }

  /**
   * {@inheritdoc}
   */
  public function settingsForm(array $form, FormStateInterface $form_state) {
    $elements = parent::settingsForm($form, $form_state);
    $elements[] = [
      '#type' => 'markup',
      '#markup' => t('The map will be rendered using the upload Geojson file.'),
    ];
    return $elements;
  }

  /**
   * {@inheritdoc}
   */
  public function visualise(EkanResourceEntity $resource, array $settings = NULL) {
    $elements = NULL;
    if (!$resource->get('upload')->getEntity()) {
      return NULL;
    }
    if (!$resource->get('visualisation')->value) {
      return NULL;
    }
    /** @var \Drupal\taxonomy\TermInterface $format */
    $format = $resource->get('resource_format')->entity;
    if ($format->getName() == 'geojson') {
      /** @var \Drupal\file\Entity\File $file */
      $file = $resource->get('upload')->entity;
      if ($file instanceof FileInterface && $this->applicableExtension($file)) {
        $height = '400px';
        $id = 'visualisation-csv-leaflet-map-' . $resource->id();
        $elements = [
          '#type' => 'container',
          '#attributes' => [
            'class' => [$id],
            'id' => [$id],
            'style' => "height: $height;",
          ],
        ];
        $map = leaflet_map_get_info('OSM Mapnik');
        $map['id'] = $id;
        $map['geofield_cardinality'] = 1;
        $this->setAdditionalMapOptions($map, []);
        $features[] = [
          'type' => 'url',
          'url' => file_create_url($file->getFileUri()),
          'popup' => $resource->label(),
        ];
        $elements['map'] = $this->leafletRenderMap($map, $features, $height);
      }
    }

    return $elements;
  }

  /**
   * Helper function to read csv and format data for bar chart.
   */
  public function readCsv($filePath, $settings) {
    $rowcount = 0;
    $table_data = [];
    $formatted_data = [];
    $handle = fopen($filePath, "r");
    while ($csvcontents = fgetcsv($handle, 0)) {
      $table_data[$rowcount] = [];
      $columnindex = 0;
      foreach ($csvcontents as $column) {
        $table_data[$rowcount][$columnindex] = $column;
        $columnindex++;
      }
      $rowcount++;
    }
    if ($settings['has_header']) {
      unset($table_data[0]);
    }

    foreach ($table_data as $data) {
      $formatted_data[] = [
        'type' => 'point',
        'lat' => $data[$settings['lat_column'] - 1],
        'lon' => $data[$settings['long_column'] - 1],
      ];
    }
    fclose($handle);
    return $formatted_data;
  }

  /**
   * Load all Leaflet required client files and return markup for a map.
   *
   * @param array $map
   *   The map settings array.
   * @param array $features
   *   The features array.
   * @param string $height
   *   The height value string.
   *
   * @return array
   *   The leaflet_map render array.
   */
  public function leafletRenderMap(array $map, array $features = [], $height = '400px') {
    $map_id = isset($map['id']) ? $map['id'] : Html::getUniqueId('leaflet_map');
    $attached_libraries = [
      'ekan_visuals/geojson_visualisation',
      'ekan_visuals/leaflet.ajax',
      'leaflet/general',
    ];
    $settings[$map_id] = [
      'mapid' => $map_id,
      'map' => $map,
      // JS only works with arrays, make sure we have one with numeric keys.
      'features' => array_values($features),
    ];
    return [
      '#theme' => 'leaflet_map',
      '#map_id' => $map_id,
      '#height' => $height,
      '#map' => $map,
      '#attached' => [
        'library' => $attached_libraries,
        'drupalSettings' => [
          'geojsonleaflet' => $settings,
        ],
      ],
    ];
  }

  /**
   * Check if the file extension is geojson.
   */
  public function applicableExtension(FileInterface $file) {
    $regex = '/\\.(' . preg_replace('/ +/', '|', preg_quote('geojson')) . ')$/i';
    if (preg_match($regex, $file->getFilename())) {
      return TRUE;
    }
    return FALSE;
  }

}
